/**************************************************************************
*   Copyright (C) 2011 by Claus Peter Blunck                              *
*   ply2@scienide.de                                                      *
*                                                                         *
*   Permission is hereby granted, free of charge, to any person obtaining *
*   a copy of this software and associated documentation files (the       *
*   "Software"), to deal in the Software without restriction, including   *
*   without limitation the rights to use, copy, modify, merge, publish,   *
*   distribute, sublicense, and/or sell copies of the Software, and to    *
*   permit persons to whom the Software is furnished to do so, subject to *
*   the following conditions:                                             *
*                                                                         *
*   The above copyright notice and this permission notice shall be        *
*   included in all copies or substantial portions of the Software.       *
*                                                                         *
*   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,       *
*   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF    *
*   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.*
*   IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR     *
*   OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, *
*   ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR *
*   OTHER DEALINGS IN THE SOFTWARE.                                       *
**************************************************************************/

#include <windows.h>
#include <stdio.h>
#include <stdlib.h>

void showusage(FILE* f) {
    fprintf(f,"tousb001: Send a file (as raw data) to a windows printer.\r\n");
    fprintf(f,"Version: 1.0 (2011-01-08)\r\n");
    fprintf(f,"Credits: Code by ply2/Scienide - visit www.scienide.de\r\n");
    fprintf(f,"Usage: tousb001 [PRINTER] [FILENAME]\r\n");
    fprintf(f,"Example: tousb001 \"LP 2844\" C:\\tmp\\poop.txt\r\n");
}
int main ( int argc, char *argv[] ) {

    HANDLE  hPrinter;
    DWORD dwPrintjob;
    BOOL bPage;
    FILE* fin;
    int progrc;

    char buffer[65536+10];
    char* printername;
    char* filename;
    int cnt;
    DOC_INFO_1 docinfo;

    hPrinter=NULL;
    bPage=FALSE;
    fin=NULL;
    dwPrintjob = 0;
    progrc = EXIT_FAILURE;

	// int i;for (i=0; i < argc; i++ ) printf( "arg %d: %s\n", i, argv[i] );
    if (argc != 3) {
        fprintf(stderr,"Invalid argment count. :-(\r\n");
        showusage(stderr);
        return EXIT_FAILURE;
    }

    printername = argv[1];
    filename = argv[2];

    fin = fopen(filename,"rb");
    if (fin == NULL) {
        fprintf(stderr,"Failed to open Infile '%s'. :-(\r\n",filename);
        goto errexit;
    }
    if (OpenPrinter(printername, &hPrinter, NULL) == FALSE) {
        hPrinter = NULL;
        fprintf(stderr,"Failed to open printer '%s' (Errcode %x). :-(\r\n",printername, (unsigned int)GetLastError());
        goto errexit;
    }
    if (hPrinter == NULL) {
        fprintf(stderr,"Failed to open printer '%s' (Errcode OK but hPrinter = NULL). :-(\r\n",printername);
        goto errexit;
    }

    memset(&docinfo,0,sizeof(docinfo));
    docinfo.pDocName = filename;
    docinfo.pOutputFile = NULL;
    docinfo.pDatatype = "RAW";

    dwPrintjob = StartDocPrinter(hPrinter, 1,(LPBYTE)&docinfo);
    if (dwPrintjob == 0) {
        fprintf(stderr,"Failed to start print job (StartDocPrinter Errcode %x). :-(\r\n", (unsigned int)GetLastError());
        goto errexit;
    }
    bPage = StartPagePrinter(hPrinter);
    if (bPage == FALSE) {
        fprintf(stderr,"Failed to start print job (StartPagePrinter Errcode %x). :-(\r\n", (unsigned int)GetLastError());
        goto errexit;
    }
    do {
        DWORD dwNumwritten;
        cnt = fread( (void*)buffer, 1, 65536, fin);
        buffer[65536] = 0;
        // printf(buffer);
        BOOL bSuccess = WritePrinter(hPrinter, (LPVOID)buffer, (DWORD)cnt, &dwNumwritten);
        if (!bSuccess) {
            fprintf(stderr,"Failed to write to print job page (Errcode %x). :-(\r\n", (unsigned int)GetLastError());
            goto errexit;
        }
    } while (cnt == 65536);
    progrc = EXIT_SUCCESS;

    errexit:
    if (hPrinter != NULL && dwPrintjob != 0 && bPage != FALSE) EndPagePrinter(hPrinter);
    if (hPrinter != NULL && dwPrintjob != 0) EndDocPrinter(hPrinter);
    if (hPrinter != NULL) ClosePrinter(hPrinter);
    if (fin != NULL) fclose(fin);
    return progrc;
}
